#include<LEDA/graph/graph.h>
#include<LEDA/core/array.h>
#include<LEDA/numbers/integer.h>
#include<iostream>
using namespace leda;
using namespace std;

int algorithm(graph &G,array<int> &pol);
int complete_g(graph &G);
int calculation(graph &G,array<int> &pol);
edge add_e(graph &G);
void contraction(graph &G,edge e);

int main()
{
  graph G;
  G.read("graph.gw");

  G.make_undirected();
  int v = G.number_of_nodes();
  array<int> pol(v+1);
  for(int i=0;i<=v;i++)
    pol[i]=0;

  algorithm(G,pol);

  for(int j=v;j>0;j--){
    cout << pol[j] << "x^"<< j ;
    if(j != 1)
      if(pol[j-1] > 0)
	cout << " + " ;
      else{
	cout << " - " ;
	pol[j-1] = pol[j-1] * (-1);
      }
  }
  cout << endl;
}

int algorithm(graph &G,array<int> &pol)
{

  if( complete_g(G) ){
    calculation(G,pol);
    return 0;
  }

  graph G_2 = G;
  edge e = add_e(G_2);

  graph G_1 = G_2;
  contraction(G_2,e);

  int v = G.number_of_nodes();

  array<int> pol_1(v+1);
  array<int> pol_2(v+1);

  for(int i=0;i<=v;i++){
    pol_1[i]=0;
    pol_2[i]=0;
  }

  algorithm(G_1,pol_1);
  algorithm(G_2,pol_2);

  for(int i=0;i<=v;i++)
    pol[i]= pol[i]+pol_1[i]+pol_2[i];

  return 0;
}

int complete_g(graph &G)
{
  node v;
  int i = G.number_of_nodes();

  int sum = 0;
  for(i=i-1;i>0;i--)
    sum += i;

  if(G.number_of_edges() == sum )
    return 1;
  else return 0;
}

int calculation(graph &G,array<int> &pol)
{
  int n = G.number_of_nodes();

  array<int> w(n+1);
  pol[0]=0;
  int j =1;
  pol[j]=1;
  int k;
  w = pol;
  for(j=2;j<=n;j++){
    for(k=1;k<=j;k++){
      if(k==j)
	pol[k] = 1; 
      else{  
	pol[k] = pol[k]*(j-1)*(-1);
	pol[k] += w[k-1];
      }
    }
    w = pol;
  }

  /*
  for(int j=n;j>0;j--){
    cout << pol[j] << "x_"<< j ;
    if(j != 1)
      cout << " + " ;
  }
  cout << endl;
  */
  return 0;
}

edge add_e(graph &G)
{
  int n = G.number_of_nodes();
  node v;
  node_array<int> n_a(G,0);

  for(;;){
    v = G.choose_node();
    if(G.degree(v) < n-1)
      break;
  }

  n_a[v]=1;
  node w;
  forall_adj_nodes(w,v)
    n_a[w]=1;

  forall_nodes(w,G)
    if(n_a[w] == 0)
      break;

  edge e = G.new_edge(v,w);
  return e;
}

void contraction(graph &G,edge e)
{
  G.make_undirected();

  node v,w,x;

  v = G.source(e);
  w = G.opposite(v,e);

  node_array<int> node_ord(G,0);
 
  forall_adj_nodes(x,v)
    node_ord[x]=1;
  forall_adj_nodes(x,w)
    node_ord[x]=1;
  node_ord[v]=0;
  node_ord[w]=0;

  node new_v=G.new_node();
  forall_nodes(x,G)
    if(x != new_v)
      if(node_ord[x] == 1)
	G.new_edge(new_v,x);
  G.del_node(v);
  G.del_node(w);

}
